#------------------------------------------------------------------------------#
#
#
cmake_minimum_required(VERSION 3.8 FATAL_ERROR)

cmake_policy(SET CMP0012 NEW)

get_filename_component(PTL_CMAKE_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

include(${CMAKE_CURRENT_LIST_DIR}/PTLConfigVersion.cmake)
include(CMakeFindDependencyMacro)

#------------------------------------------------------------------------------#
#   available components
#
set(_PTL_AVAILABLE_COMPONENTS )

if(ON)
    set(_PTL_SHARED_COMPONENT shared)
endif()

if(ON)
    set(_PTL_STATIC_COMPONENT static)
endif()

set(_PTL_PREFER_SHARED ON)

if(BUILD_SHARED_LIBS OR "shared" IN_LIST PTL_FIND_COMPONENTS)
    set(_PTL_PREFER_SHARED ON)
endif()

if("static" IN_LIST PTL_FIND_COMPONENTS)
    set(_PTL_PREFER_SHARED OFF)
endif()

if(DEFINED BUILD_SHARED_LIBS AND NOT BUILD_SHARED_LIBS)
    set(_PTL_PREFER_SHARED OFF)
endif()

if(_PTL_PREFER_SHARED)
    # if parent package set BUILD_SHARED_LIBS, prefer shared library
    list(APPEND _PTL_AVAILABLE_COMPONENTS
        ${_PTL_SHARED_COMPONENT} ${_PTL_STATIC_COMPONENT})
else()
    # if parent package has not set BUILD_SHARED_LIBS, prefer static library
    list(APPEND _PTL_AVAILABLE_COMPONENTS
        ${_PTL_STATIC_COMPONENT} ${_PTL_SHARED_COMPONENT})
endif()

unset(_PTL_SHARED_COMPONENT)
unset(_PTL_STATIC_COMPONENT)

#------------------------------------------------------------------------------#
#   package initialization
#

####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was PTLConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

# Use original install prefix when loaded through a "/usr move"
# cross-prefix symbolic link such as /lib -> /usr/lib.
get_filename_component(_realCurr "${CMAKE_CURRENT_LIST_DIR}" REALPATH)
get_filename_component(_realOrig "/usr/lib/arm-linux-gnueabi/PTL" REALPATH)
if(_realCurr STREQUAL _realOrig)
  set(PACKAGE_PREFIX_DIR "/usr")
endif()
unset(_realOrig)
unset(_realCurr)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################


#------------------------------------------------------------------------------#
#   configure lib and include
#
set_and_check(PTL_INCLUDE_DIR "${PACKAGE_PREFIX_DIR}/include")
set_and_check(PTL_LIB_DIR "${PACKAGE_PREFIX_DIR}/lib/arm-linux-gnueabi")

include(${PTL_CMAKE_DIR}/PTLTargets.cmake)
check_required_components(PTL)

# get target
foreach(_COMPONENT ${_PTL_AVAILABLE_COMPONENTS})
    # set the library
    if(NOT PTL_LIBRARY AND TARGET PTL::ptl-${_COMPONENT})
        set(PTL_LIBRARY PTL::ptl-${_COMPONENT})
    endif()
    if(TARGET PTL::ptl-${_COMPONENT})
        set(PTL_${_COMPONENT}_FOUND ON)
        set(PTL_${_COMPONENT}_LIBRARY PTL::ptl-${_COMPONENT})
        if(WIN32)
            target_compile_definitions(PTL::ptl-${_COMPONENT} PUBLIC _PTL_ARCHIVE)
        endif()
    endif()
endforeach()
unset(_PTL_AVAILABLE_COMPONENTS)

# save original module path
set(_MODULE_PATH ${CMAKE_MODULE_PATH})

if(TRUE)
    set(CMAKE_THREAD_PREFER_PTHREAD ON)
    set(THREADS_PREFER_PTHREAD_FLAG ON)
    find_package(Threads REQUIRED)
endif()

if(TRUE)
    set(TBB_ROOT_DIR /usr)
    set(CMAKE_MODULE_PATH ${CMAKE_CURRENT_LIST_DIR}/Modules ${CMAKE_MODULE_PATH})
    find_package(TBB 2020.3 REQUIRED)
endif()

# restore original module path
set(CMAKE_MODULE_PATH ${_MODULE_PATH})


#------------------------------------------------------------------------------#
# internal PTL include directory
#
set(PTL_INCLUDE_DIRS ${PTL_INCLUDE_DIR})


#------------------------------------------------------------------------------#
#   CXX standard
#
if(DEFINED CMAKE_CXX_STANDARD)
    if("${CMAKE_CXX_STANDARD}" VERSION_LESS 11)
        set(CMAKE_CXX_STANDARD 11)
        set(CMAKE_CXX_STANDARD_REQUIRED ON)
    endif()
else(DEFINED CMAKE_CXX_STANDARD)
    set(CMAKE_CXX_STANDARD 11)
endif(DEFINED CMAKE_CXX_STANDARD)

set(CMAKE_CXX_STANDARD_REQUIRED ON)


#------------------------------------------------------------------------------#
#   link target
#
set(PTL_LIBRARIES ${PTL_LIBRARY})


#------------------------------------------------------------------------------#
#   Definitions
#
set(PTL_USE_TBB ON)
set(PTL_USE_GPU OFF)
set(PTL_USE_CUDA OFF)

if(NOT _PTL_INSTALL_TREE)
    list(APPEND PTL_LIBRARIES "")
endif()

if(NOT TARGET PTL::ptl)
    add_library(PTL::ptl INTERFACE IMPORTED)
    target_include_directories(PTL::ptl INTERFACE ${PTL_INCLUDE_DIRS})
    target_link_libraries(PTL::ptl INTERFACE ${PTL_LIBRARIES})
    set_target_properties(PTL::ptl PROPERTIES
        INTERFACE_CXX_STANDARD 11
        INTERFACE_CXX_STANDARD_REQUIRED ON)
    if(WIN32)
        target_compile_definitions(PTL::ptl INTERFACE _PTL_ARCHIVE)
    endif()
    foreach(_OPT PTL_USE_TBB PTL_USE_GPU PTL_USE_CUDA)
        if(${_OPT})
            target_compile_definitions(PTL::ptl INTERFACE ${_OPT})
        endif()
    endforeach()
endif()

#------------------------------------------------------------------------------#
#   find_package variables
#
include(FindPackageHandleStandardArgs)

find_package_handle_standard_args(
    PTL
    VERSION_VAR
        PTL_VERSION
    REQUIRED_VARS
        PTL_CMAKE_DIR
        PTL_LIBRARIES
        PTL_INCLUDE_DIRS
    HANDLE_COMPONENTS
)
